import 'package:ocg_app/ocg_app.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:uuid/uuid.dart';

import 'event.dart';
import 'state.dart';

import '../../repository/chat.dart';
import '../../models/chat.dart';

class ChatCreateBloc extends Bloc<ChatCreateEvent, ChatCreateState> {
  final ChatRepository chatRepository;
  ChatCreateBloc(this.chatRepository) : super(ChatCreateIdleState());

  @override
  Stream<ChatCreateState> mapEventToState(ChatCreateEvent event) async* {
    if (event is ChatCreateStartEvent) {
      yield ChatCreateLoadingState();
      String id = Uuid().v4();
      Chat newChat = event.chat.copyWith(
        id: id,
      );
      AppResponse<String> response;
      if (newChat.isGroup ?? true) {
        response = await chatRepository.createChat(
          newChat,
        );
      } else {
        debugPrint("Start Chat");
        response = await chatRepository.startChat(newChat.users[0]);
      }
      if (response.isSuccess) {
        AppResponse<Chat> chatReponse =
            await chatRepository.getChatDetails(response.data);
        if (chatReponse.isSuccess)
          yield ChatCreateSuccessState(chatReponse.data);
        else {
          yield ChatCreateErrorState(chatReponse.error);
        }
        yield ChatCreateIdleState();
      } else {
        yield ChatCreateErrorState(response.error);
        yield ChatCreateIdleState();
      }
    }
  }
}
