import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_chat/src/bloc/chat_detail/event.dart';
import 'package:ocg_chat/src/bloc/chat_detail/state.dart';
import 'package:ocg_chat/src/models/chat.dart';
import 'package:ocg_chat/src/models/message.dart';
import 'package:ocg_chat/src/repository/chat.dart';
import 'package:ocg_chat/src/repository/message.dart';

class ChatDetailBloc extends Bloc<ChatDetailEvent, ChatDetailState> {
  final ChatRepository chatRepository;
  final MessageRepository messageRepository;
  ChatDetailBloc(this.chatRepository, this.messageRepository)
      : super(ChatDetailIdleState());

  @override
  Stream<ChatDetailState> mapEventToState(ChatDetailEvent event) async* {
    if (event is ChatDetailRefreshEvent) {
      Chat chat;
      List<Message> sharedMedia;
      String error;
      await Future.wait([
        Future(() async {
          AppResponse<Chat> chatResponse =
              await chatRepository.getChatDetails(event.chatId);
          if (chatResponse.isSuccess) {
            chat = chatResponse.data;
          } else {
            error = chatResponse.error ?? error;
          }
        }),
        Future(() async {
          AppResponse<List<Message>> messageResponse =
              await messageRepository.getSharedFilesList(event.chatId);
          if (messageResponse.isSuccess) {
            sharedMedia = messageResponse.data;
          } else {
            error = messageResponse.error ?? error;
          }
        }),
      ]);
      if (error == null) {
        yield ChatDetailLoadedState(
          event.chatId,
          chat,
          sharedMedia,
        );
      } else {
        yield ChatDetailErrorState(event.chatId, error);
      }
    }
  }
}
