import 'package:ocg_app/ocg_app.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_user/ocg_user.dart';

import 'event.dart';
import 'state.dart';

import '../../repository/chat.dart';
import '../../models/signal.dart';

class ChatSignalBloc extends Bloc<ChatSignalEvent, ChatSignalState> {
  final ChatRepository chatRepository;
  final UserRepository userRepository;

  ChatSignalBloc(
    this.chatRepository,
    this.userRepository,
  ) : super(ChatSignalLoadedState([]));
  @override
  Stream<ChatSignalState> mapEventToState(ChatSignalEvent event) async* {
    if (event is ChatSignalAddEvent) {
      await chatRepository.addSignal(event.signal);
    }
    if (state is ChatSignalLoadedState) {
      List<ChatSignal> signals =
          List<ChatSignal>.from((state as ChatSignalLoadedState).signals);
      if (event is ChatSignalReceiveEvent) {
        if (!signals.contains(event.signal)) {
          AppResponse<User> userResponse =
              await userRepository.getUser(event.signal.senderId);
          if (userResponse.isSuccess) {
            signals.add(event.signal.copyWith(sender: userResponse.data));
          }
        }
        yield ChatSignalLoadedState(signals);
      }
      if (event is ChatSignalDeleteEvent) {
        if (signals.contains(event.signal)) {
          signals.remove(event.signal);
        }
        yield ChatSignalLoadedState(signals);
      }
    }
  }
}
