import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_chat/ocg_chat.dart';
import 'package:ocg_chat/src/bloc/chat_update/event.dart';
import 'package:ocg_chat/src/bloc/chat_update/state.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

class ChatUpdateBloc extends Bloc<ChatUpdateEvent, ChatUpdateState> {
  final ChatRepository chatRepository;
  ChatUpdateBloc(this.chatRepository) : super(ChatUpdateInitialState());

  @override
  Stream<ChatUpdateState> mapEventToState(ChatUpdateEvent event) async* {
    AppResponse<bool> response;
    yield ChatUpdateLoadingState(event);
    if (event is ChatUpdateKickEvent) {
      response =
          await chatRepository.removeGroupUser(event.chatId, event.userId);
    } else if (event is ChatUpdateRenameEvent) {
      response = await chatRepository.renameGroup(event.chatId, event.name);
    } else if (event is ChatUpdateAdminEvent) {
      response = await chatRepository.makeGroupAdmin(event.chatId, event.admin);
    } else if (event is ChatUpdateAddListEvent) {
      response = await chatRepository.addGroupUser(event.chatId, event.users);
    } else if (event is ChatUpdateMuteEvent) {
      response = await chatRepository.muteChat(event.chatId, mute: event.muted);
    } else if (event is ChatUpdateMetadataEvent) {
      response =
          await chatRepository.updateMetadata(event.chatId, event.metadata);
    } else if (event is ChatUpdateLeaveEvent) {
      response = await chatRepository.leaveGroup(
        event.chatId,
      );
    } else if (event is ChatUpdateClearEvent) {
      response = await chatRepository.clearChat(
        event.chatId,
      );
    }
    if (response != null) {
      if (response.isSuccess) {
        yield ChatUpdateSuccessState(event);
      } else {
        yield ChatUpdateErrorState(event, response.error);
      }
    }
  }
}
