import 'package:json_annotation/json_annotation.dart';
import 'package:ulid/ulid.dart';
import 'package:ocg_user/ocg_user.dart';

import 'message.dart';

part 'chat.g.dart';

@JsonSerializable(explicitToJson: true)
class Chat {
  @JsonKey(name: "chat_id")
  final String id;
  final List<String> users;
  @JsonKey(name: "is_group", defaultValue: false)
  final bool isGroup;
  @JsonKey(defaultValue: [])
  final List<String> admins;
  @JsonKey(
    name: "group_name",
  )
  final String groupName;
  final String groupPic;
  @JsonKey(name: "unread_msg_count")
  final int unReadMsgCount;
  @JsonKey(defaultValue: false)
  final bool muted;
  @JsonKey(name: "last_read_msg_id")
  final String lastMsgId;
  @JsonKey(name: "last_msg")
  final Message lastMsg;
  @JsonKey(ignore: true)
  final User friend;
  final Map<String, dynamic> metadata;

  Chat(
      this.id,
      this.users,
      this.isGroup,
      this.admins,
      this.groupName,
      this.muted,
      this.lastMsgId,
      this.lastMsg,
      this.groupPic,
      this.unReadMsgCount,
      this.metadata)
      : friend = null;

  Chat.named(
      {this.id,
      this.users,
      this.isGroup,
      this.admins,
      this.groupName,
      this.groupPic,
      this.muted,
      this.lastMsgId,
      this.lastMsg,
      this.friend,
      this.unReadMsgCount,
      this.metadata});

  Map<String, dynamic> toJson() => _$ChatToJson(this);

  factory Chat.fromJson(json) => _$ChatFromJson(json);

  String get title => ((isGroup ?? false) ? groupName : friend?.name);

  DateTime get recordedAt => lastMsg?.recordedAt;

  Chat copyWith(
      {String id,
      List<String> users,
      bool isGroup,
      List<String> admins,
      String groupName,
      bool muted,
      String lastMsgId,
      Message lastMsg,
      String groupPic,
      User friend,
      int unReadMsgCount,
      Map<String, dynamic> metadata}) {
    return Chat.named(
        id: id ?? this.id,
        admins: admins ?? this.admins,
        groupName: groupName ?? this.groupName,
        groupPic: groupPic ?? this.groupPic,
        isGroup: isGroup ?? this.isGroup,
        lastMsg: lastMsg ?? this.lastMsg,
        lastMsgId: lastMsgId ?? this.lastMsgId,
        muted: muted ?? this.muted,
        users: users ?? this.users,
        friend: friend ?? this.friend,
        unReadMsgCount: unReadMsgCount ?? this.unReadMsgCount,
        metadata: metadata ?? this.metadata);
  }

  DateTime get lastReadMegRecordedAt {
    try {
      return (lastMsgId != null
              ? DateTime.fromMillisecondsSinceEpoch(
                  Ulid.parse(lastMsgId).toMillis())
              : null) ??
          DateTime(1971);
    } catch (e) {
      return DateTime(1971);
    }
  }
}
