import 'package:ocg_chat/src/models/message.dart';
import 'package:json_annotation/json_annotation.dart';

part 'socket.g.dart';

class SocketType {
  // Chat
  static const String MSG_READ = "MSG_READ";
  static const String NEW_MSG = "NEW_MSG";
  static const String MSG_DELETE = "MSG_DELETE";
  static const String NEW_CHAT = "NEW_CHAT";
  static const String GROUP_INFO_CHANGED = "GROUP_INFO_CHANGED";
  static const String CLEAR_CHAT = "CLEAR_CHAT";
  // Meeting
  static const String NEW_MEETING_MSG = "NEW_MEETING_MSG";
  static const String JOIN_MEETING_REQUEST = "JOIN_MEETING";
  static const String MEETING_INFO_CHANGE = "MEETING_INFO_CHANGE";
  static const String END_MEETING = "END_MEETING";
  static const String MEETING_MEDIA_STATUS_CHANGED =
      "MEETING_MEDIA_STATUS_CHANGED";
  static const String MEETING_KICK_USER = "MEETING_KICK_USER";
  // Broadcast
  static const String NEW_BROADCAST_MSG = "NEW_BROADCAST_MSG";
  static const String END_BROADCAST = "END_BROADCAST";
  static const String JOIN_BROADCAST = "JOIN_BROADCAST";
}

abstract class SocketData {
  final String type;

  SocketData(this.type);
}

abstract class SocketChatData extends SocketData {
  SocketChatData(String type) : super(type);
}

@JsonSerializable(explicitToJson: true)
class SocketMessageData extends SocketChatData {
  final Message message;

  SocketMessageData(this.message, String type) : super(type);

  SocketMessageData.named({this.message, String type}) : super(type);

  factory SocketMessageData.fromJson(json) => _$SocketMessageDataFromJson(json);

  Map<String, dynamic> toJson() => _$SocketMessageDataToJson(this);

  SocketMessageData copyWith({Message message}) {
    return SocketMessageData.named(message: message ?? this.message);
  }
}

@JsonSerializable(explicitToJson: true)
class SocketReadData extends SocketChatData {
  @JsonKey(name: "chat_id")
  final String chatId;
  @JsonKey(name: "msg_id")
  final String msgId;

  SocketReadData(this.chatId, this.msgId, String type) : super(type);

  SocketReadData.named({this.chatId, this.msgId, String type}) : super(type);

  factory SocketReadData.fromJson(json) => _$SocketReadDataFromJson(json);

  Map<String, dynamic> toJson() => _$SocketReadDataToJson(this);
}

@JsonSerializable(explicitToJson: true)
class SocketMsgDeleteData extends SocketChatData {
  @JsonKey(name: "chat_id")
  final String chatId;
  @JsonKey(name: "msg_id")
  final String msgId;
  @JsonKey(name: "user_id")
  final String userId;

  SocketMsgDeleteData(this.chatId, this.msgId, this.userId)
      : super(SocketType.MSG_DELETE);

  Map<String, dynamic> toJson() => _$SocketMsgDeleteDataToJson(this);

  factory SocketMsgDeleteData.fromJson(json) =>
      _$SocketMsgDeleteDataFromJson(json);
}

@JsonSerializable(explicitToJson: true)
class SocketNewChatData extends SocketChatData {
  @JsonKey(name: "chat_id")
  final String chatId;

  SocketNewChatData(
    this.chatId,
  ) : super(SocketType.NEW_CHAT);

  Map<String, dynamic> toJson() => _$SocketNewChatDataToJson(this);

  factory SocketNewChatData.fromJson(json) => _$SocketNewChatDataFromJson(json);
}

@JsonSerializable(explicitToJson: true)
class SocketGroupInfoChanged extends SocketChatData {
  @JsonKey(name: "chat_id")
  final String chatId;

  SocketGroupInfoChanged(
    this.chatId,
  ) : super(SocketType.GROUP_INFO_CHANGED);

  Map<String, dynamic> toJson() => _$SocketGroupInfoChangedToJson(this);

  factory SocketGroupInfoChanged.fromJson(json) =>
      _$SocketGroupInfoChangedFromJson(json);
}

@JsonSerializable(explicitToJson: true)
class SocketClearData extends SocketChatData {
  @JsonKey(name: "chat_id")
  final String chatId;

  SocketClearData(this.chatId, String type) : super(type);

  Map<String, dynamic> toJson() => _$SocketClearDataToJson(this);

  factory SocketClearData.fromJson(json) => _$SocketClearDataFromJson(json);
}

@JsonSerializable(explicitToJson: true)
class SocketMeetingJoinData extends SocketData {
  @JsonKey(name: "meeting_id")
  final String meetingId;
  @JsonKey(name: "user_id")
  final String userId;
  final String type;

  SocketMeetingJoinData(this.meetingId, this.userId, this.type) : super(type);

  factory SocketMeetingJoinData.fromJson(json) =>
      _$SocketMeetingJoinDataFromJson(json);

  Map<String, dynamic> toJson() => _$SocketMeetingJoinDataToJson(this);
}

@JsonSerializable(explicitToJson: true)
class SocketMeetingMediaStatusData extends SocketData {
  @JsonKey(name: "meeting_id")
  final String meetingId;
  @JsonKey(name: "media_type")
  final String mediaType;
  final String status;

  SocketMeetingMediaStatusData(
      this.meetingId, this.mediaType, this.status, String type)
      : super(type);

  factory SocketMeetingMediaStatusData.fromJson(json) =>
      _$SocketMeetingMediaStatusDataFromJson(json);

  Map<String, dynamic> toJson() => _$SocketMeetingMediaStatusDataToJson(this);
}

@JsonSerializable(explicitToJson: true)
class SocketMeetingKickUserData extends SocketData {
  @JsonKey(name: "meeting_id")
  final String meetingId;

  SocketMeetingKickUserData(this.meetingId, String type) : super(type);

  factory SocketMeetingKickUserData.fromJson(json) =>
      _$SocketMeetingKickUserDataFromJson(json);

  Map<String, dynamic> toJson() => _$SocketMeetingKickUserDataToJson(this);
}

@JsonSerializable(explicitToJson: true)
class SocketMeetingInfoChangeData extends SocketData {
  @JsonKey(name: "meeting_id")
  final String meetingId;

  SocketMeetingInfoChangeData(this.meetingId, String type) : super(type);

  factory SocketMeetingInfoChangeData.fromJson(json) =>
      _$SocketMeetingInfoChangeDataFromJson(json);

  Map<String, dynamic> toJson() => _$SocketMeetingInfoChangeDataToJson(this);
}

///
/// Bbroadcast
///

@JsonSerializable(explicitToJson: true)
class SocketBroadcastEndData extends SocketData {
  @JsonKey(name: "broadcast_id")
  final String broadcastId;

  SocketBroadcastEndData(this.broadcastId, String type) : super(type);

  Map<String, dynamic> toJson() => _$SocketBroadcastEndDataToJson(this);

  factory SocketBroadcastEndData.fromJson(json) =>
      _$SocketBroadcastEndDataFromJson(json);
}

@JsonSerializable(explicitToJson: true)
class SocketBroadcastJoinData extends SocketData {
  @JsonKey(name: "broadcast_id")
  final String broadcastId;
  @JsonKey(name: "user_id")
  final String userId;

  SocketBroadcastJoinData(this.broadcastId, this.userId)
      : super(SocketType.JOIN_BROADCAST);

  Map<String, dynamic> toJson() => _$SocketBroadcastJoinDataToJson(this);

  factory SocketBroadcastJoinData.fromJson(json) =>
      _$SocketBroadcastJoinDataFromJson(json);
}
