import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_chat/src/models/message.dart';
import 'package:ocg_user/ocg_user.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

class MessageDetailsScreen extends StatefulWidget {
  final Map<String, MessageReceiveData> receiveData;
  MessageDetailsScreen({Key key, this.receiveData}) : super(key: key);

  @override
  _MessageDetailsScreenState createState() => _MessageDetailsScreenState();
}

class _MessageDetailsScreenState extends State<MessageDetailsScreen> {
  List<User> users;
  bool isLoading;

  @override
  void initState() {
    super.initState();
    isLoading = true;
    users = [];
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    loadData();
  }

  Future<void> loadData() async {
    UserRepository userRepository =
        BlocProvider.of<UserListBloc>(context).userRepository;
    if (widget.receiveData != null) {
      users = await Future.wait(widget.receiveData.keys
          .map((e) async {
            AppResponse<User> response = await userRepository.getUser(e);
            if (response.isSuccess) {
              return response.data;
            }
            return null;
          })
          .where((element) => element != null)
          .toList());
    }
    if (mounted) {
      setState(() {
        isLoading = false;
      });
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text("Message Info"),
      ),
      body: isLoading
          ? Center(
              child: CircularProgressIndicator(),
            )
          : ListView.builder(
              itemBuilder: (context, index) {
                User user = users[index];
                return ListTile(
                  leading: CircleAvatar(
                    backgroundImage: NetworkImage(user.profilePic),
                  ),
                  title: Text("${user.name}"),
                  subtitle: Text(
                      "${DateHelper.beautifyDate(widget.receiveData[user.id].recordedAt)}"),
                );
              },
              itemCount: users.length,
              shrinkWrap: true,
            ),
    );
  }
}
