import 'package:ocg_chat/src/bloc/chat/bloc.dart';
import 'package:ocg_chat/src/bloc/chat/state.dart';
import 'package:ocg_chat/src/bloc/chat_signal/bloc.dart';
import 'package:ocg_chat/src/bloc/chat_signal/event.dart';
import 'package:ocg_chat/src/models/chat.dart';
import 'package:ocg_chat/src/models/message.dart';
import 'package:ocg_chat/src/models/signal.dart';
import 'package:ocg_chat/src/widgets/message_app_bar.dart';
import 'package:ocg_login/ocg_login.dart';
import 'package:ocg_user/ocg_user.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';

import '../bloc/message/bloc.dart';
import '../bloc/message/state.dart';
import '../bloc/message/event.dart';

import '../widgets/message_input_widget.dart';
import '../widgets/message_list_view.dart';

class MessageScreen extends StatefulWidget {
  MessageScreen({Key key}) : super(key: key);

  @override
  _MessageScreenState createState() => _MessageScreenState();
}

class _MessageScreenState extends State<MessageScreen> {
  ScrollController scrollController;

  String chatId;

  List<Message> messages;

  @override
  void initState() {
    super.initState();
    scrollController = ScrollController();
    messages = [];
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    var arguments = ModalRoute.of(context).settings.arguments;
    if (arguments is Map) {
      chatId = arguments["chatId"];
      if (chatId != null) {
        setState(() {});
      }
    }
    if (chatId == null) {
      return;
    }
    BlocProvider.of<MessageBloc>(context).add(MessageFetchEvent(chatId, null));

    Future.delayed(Duration(milliseconds: 500)).then((value) {
      if (mounted) {
        scrollToEnd();
      }
    });
  }

  Future<void> scrollToEnd() async {
    if (scrollController.hasClients)
      await scrollController.animateTo(
          scrollController.position.maxScrollExtent,
          duration: Duration(milliseconds: 200),
          curve: Curves.easeIn);
  }

  @override
  Widget build(BuildContext context) {
    return BlocListener<MessageBloc, MessageState>(
      listener: (context, state) async {
        if (state is MessagedLoadedState) {
          if (messages.length < state.messages.length) {
            await Future.delayed(Duration(milliseconds: 200));
          }
          messages = state.messages;
        }
      },
      child: Scaffold(
        body:
            BlocBuilder<LoginBloc, LoginState>(builder: (context, loginState) {
          if (!(loginState is LoginExistState)) {
            return Center(
              child: Text("User not logged in"),
            );
          }
          LoginExistState existState = loginState;
          User user = existState.user;
          return BlocBuilder<ChatBloc, ChatState>(
              builder: (context, chatState) {
            if (!(chatState is ChatLoadedState)) {
              return Center(
                child: Text("Not Loaded"),
              );
            }
            List<Chat> chats = (chatState as ChatLoadedState).chats;
            int index = chats.indexWhere((element) => element.id == chatId);
            if (index == -1) {
              return Center(
                child: Text("Not Loaded"),
              );
            }
            Chat chat = chats[index];
            return BlocBuilder<MessageBloc, MessageState>(
                builder: (context, state) {
              if (state is MessageLoadingState) {
                return Center(
                  child: CircularProgressIndicator(),
                );
              }
              if (state is MessagedLoadedState) {
                List<Message> messages = state.messages
                    .where((element) => element.chatId == chatId)
                    .toList();
                // messages = messages.map((e) {
                //   if (e.receiveData != null &&
                //       e.receiveData.length == chat.users.length - 1) {
                //     return e.copyWith(isSeen: true);
                //   }
                //   return e;
                // }).toList();
                return Column(
                  crossAxisAlignment: CrossAxisAlignment.stretch,
                  children: [
                    MesssageAppBar(
                      chat: chat,
                    ),
                    Expanded(
                        child: MessageListView(
                      scrollController: scrollController,
                      messages: messages,
                      onVisible: (message) {
                        if (!(message.isSeen ?? false) &&
                            !(message.isSelf ?? false))
                          BlocProvider.of<MessageBloc>(context)
                              .add(MessageReadEvent(message));
                      },
                      onRetry: (message) {
                        BlocProvider.of<MessageBloc>(context)
                            .add(MessageRetryEvent(message));
                      },
                    )),
                    MessageInputWidget(
                      onTyping: onTyping,
                      onSubmitted: (message) {
                        BlocProvider.of<MessageBloc>(context).add(
                            MessageSendEvent(message.copyWith(
                                chatId: chatId, senderId: user.id)));
                      },
                    )
                  ],
                );
              }
              if (state is MessageErrorState) {
                return Center(
                  child: Text("${state.error}"),
                );
              }
              return Center(
                child: Text("Error Occurred"),
              );
            });
          });
        }),
      ),
    );
  }

  void onTyping(bool isTyping) {
    if (!isTyping) {
      BlocProvider.of<ChatSignalBloc>(context).add(ChatSignalAddEvent(
          ChatSignal.named(
              channelId: chatId, message: ChatSignalType.notTyping)));
    } else if (isTyping) {
      BlocProvider.of<ChatSignalBloc>(context).add(ChatSignalAddEvent(
          ChatSignal.named(channelId: chatId, message: ChatSignalType.typing)));
    }
  }
}
