import 'package:ocg_chat/src/bloc/chat_signal/bloc.dart';
import 'package:ocg_chat/src/bloc/chat_signal/state.dart';
import 'package:ocg_chat/src/models/chat.dart';
import 'package:ocg_chat/src/models/signal.dart';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:ocg_login/ocg_login.dart';
import 'package:ocg_user/ocg_user.dart';

class MesssageAppBar extends StatelessWidget {
  final Chat chat;
  const MesssageAppBar({Key key, this.chat}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return BlocBuilder<LoginBloc, LoginState>(builder: (context, loginState) {
      if (!(loginState is LoginExistState)) {
        return Center(
          child: Text("User not logged in"),
        );
      }
      LoginExistState existState = loginState;
      User user = existState.user;
      return BlocBuilder<ChatSignalBloc, ChatSignalState>(
          builder: (context, signalState) {
        String typingSignal;

        if (signalState is ChatSignalLoadedState) {
          String value = signalState.signals
              .where((e) =>
                  e.channelId == chat.id &&
                  e.message == ChatSignalType.typing &&
                  e.senderId != user.id)
              .map((e) => "${chat.title}")
              .join(",");
          if (value.isNotEmpty) {
            typingSignal = value + " typing";
          }
        }
        return AppBar(
          title: Column(
            crossAxisAlignment: CrossAxisAlignment.start,
            children: [
              Text("${chat.title}"),
              typingSignal == null
                  ? Container(
                      width: 0,
                      height: 0,
                    )
                  : Text("$typingSignal")
            ],
          ),
        );
      });
    });
  }
}
