import 'dart:io';

import 'package:ocg_app/ocg_app.dart';
import 'package:ocg_upload/ocg_upload.dart';
import 'package:ocg_chat/src/models/message.dart';
import 'package:flutter/material.dart';
import 'package:uuid/uuid.dart';

class MessageInputWidget extends StatefulWidget {
  final Function(Message message) onSubmitted;
  final Function(bool isTyping) onTyping;
  MessageInputWidget({Key key, this.onSubmitted, this.onTyping})
      : super(key: key);

  @override
  _MessageInputWidgetState createState() => _MessageInputWidgetState();
}

class _MessageInputWidgetState extends State<MessageInputWidget> {
  UploadRepository _uploadRepository;

  TextEditingController editingController;

  @override
  void initState() {
    super.initState();
    _uploadRepository = UploadTestRepository();
    editingController = TextEditingController();
  }

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
  }

  @override
  Widget build(BuildContext context) {
    return Row(
      children: [
        Expanded(
            child: TextFormField(
          controller: editingController,
        )),
        RaisedButton(onPressed: () {
          if (!InputHelper.isBlank(editingController.text)) {
            onTextMessage(editingController.text);
            editingController.text = '';
          }
        })
      ],
    );
  }

  void onTextMessage(String content) {
    widget.onSubmitted(Message.named(
        content: TextMessage(content).toJson(), type: MessageType.text));
  }

  Future<void> onImageMessage(File file) async {
    if (file != null) {
      AppUIHelper.of(context).showLoadingDialog("Please wait..");
      AppResponse<String> uploadResponse =
          await _uploadRepository.storeFile(file, path: "${Uuid().v4()}");
      Navigator.of(context).pop();
      if (uploadResponse.isSuccess) {
        widget.onSubmitted(Message.named(
            content: ImageMessage(uploadResponse.data).toJson(),
            type: MessageType.image));
      } else {
        AppUIHelper.of(context)
            .showAlertDialog("Error", "${uploadResponse.error}");
      }
    }
  }
}
